/* conf_window.hh: Config Window header.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include <set>

#include <glibmm/ustring.h>
#include <gtkmm/window.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/box.h>
#include <gtkmm/comboboxtext.h>
#include <gtkmm/button.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/liststore.h>
#include <gtkmm/treemodelcolumn.h>
#include <gtkmm/checkbutton.h>

#include <libglademm/xml.h>

#include "boost/noncopyable.hpp"
#include "boost/filesystem/path.hpp"

namespace fs = boost::filesystem;

namespace gx2osd {

    class conf_window : private boost::noncopyable {

        public:

            conf_window();
            virtual ~conf_window();

            Gtk::Window&
            get_window();

        private:

            template< typename T >
            void
            load_widget( std::string const& name, T*& widget )
            {
                xml_->get_widget( name, widget );
                if( !widget ) {
                    throw std::runtime_error( "Could not load \"" +
                                              name + "\" widget" );
                }
            }

            void on_custom_radio_toggle();
            void on_save();
            void on_cancel();

            void skin_test( const fs::path& path );
            void add_skin( const Glib::ustring& skin );

            Glib::RefPtr< Gnome::Glade::Xml > xml_;

            Gtk::Window* window_;
            Gtk::SpinButton* delay_;
            Gtk::RadioButton* positions_;
            Gtk::ComboBox* text_pos_;
            Gtk::RadioButton* custom_;
            Gtk::HBox* pos_box_;
            Gtk::SpinButton* x_pos_;
            Gtk::SpinButton* y_pos_;
            Gtk::ComboBox* skin_;
            Gtk::Button* save_;
            Gtk::Button* cancel_;
            Gtk::CheckButton* song_change_;
            Gtk::CheckButton* play_;
            Gtk::CheckButton* pause_;
            Gtk::CheckButton* stop_;

            std::set< Glib::ustring > skin_names_;

            class skin_model : public Gtk::TreeModelColumnRecord {
                public:
                    skin_model() : Gtk::TreeModelColumnRecord(), text() {
                        add( text );
                    }
                    Gtk::TreeModelColumn< Glib::ustring > text;
            } skin_model_;
            Glib::RefPtr< Gtk::ListStore > skin_list_;

    };

}
