/* config.hh: Configuration handling header.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#ifndef GX2OSD_CONFIG_HH
#define GX2OSD_CONFIG_HH

#include "skin.hh"

#include <vector>
#include <string>

#include "boost/noncopyable.hpp"
#include "boost/program_options.hpp"
#include "boost/filesystem/path.hpp"
#include "boost/shared_ptr.hpp"

namespace po = boost::program_options;
namespace fs = boost::filesystem;
typedef std::vector< std::string > cmd_line;

namespace gx2osd {

    class config : private boost::noncopyable {

        public:

            typedef std::vector< std::string > positions_container;

            ~config();

            static config& get_instance();

            bool set_args( const cmd_line& args );

            std::string const& get_position() const;
            std::string const& get_position( positions_container::size_type pos ) const;
            positions_container::size_type get_index_of_position( std::string const& pos ) const;
            unsigned int get_xpos() const;
            unsigned int get_ypos() const;
            unsigned int get_delay() const;

            fs::path get_config_dir() const;
            const fs::path& get_pid_file() const;

            const boost::shared_ptr< gx2osd::skin >& get_skin() const;

            bool show_conf() const;

            const std::string& get_skin_name() const;

            bool get_song_change() const;
            bool get_play() const;
            bool get_pause() const;
            bool get_stop() const;

        private:
            config();
            const fs::path config_dir_;
            const fs::path config_file_;
            const fs::path pid_file_;
            
            po::options_description desc_;
            po::options_description invis_;
            po::options_description all_;
            po::variables_map vm_;

            std::string skin_name_;
            boost::shared_ptr< gx2osd::skin > skin_;
            unsigned int xpos_;
            unsigned int ypos_;
            std::string position_;
            unsigned int delay_;

            bool song_change_;
            bool play_;
            bool pause_;
            bool stop_;

            bool config_window_;
            positions_container valid_positions_;

            void parse_position( const std::string& position );
            void set_skin( const std::string& skin_name );

    };

    config& get_config();

}

#endif // GX2OSD_CONFIG_HH
