/* gx2osd.cc: On-Screen-Display (OSD) client for XMMS2.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#include <gtkmm/main.h>

#include "config.hh"
#include "main_window.hh"
#include "client.hh"
#include "conf_window.hh"

#include <iostream>

#include <boost/filesystem/fstream.hpp>
#include <boost/filesystem/operations.hpp>
namespace fs = boost::filesystem;

#include <signal.h>
#include <unistd.h>

void signal_handler( int sig ) {

    if( sig == SIGINT ) {
        Gtk::Main::quit();
    }
    else {
        gx2osd::get_window().show_all();
    }

}

struct sigaction siga;

int main( int argc, char* argv[] ) {
    
    cmd_line args( argv, argv + argc );

    Gtk::Main kit( argc, argv );

    try {
        if( gx2osd::get_config().set_args( args ) ) {

            fs::ofstream ofs( gx2osd::get_config().get_pid_file() );
            if( ofs ) {
                ofs << getpid() << std::flush;
            }

            siga.sa_handler = &signal_handler;
            sigaction( SIGUSR1, &siga, 0 );
            sigaction( SIGINT, &siga, 0 );

            gx2osd::client client;
            Gtk::Main::run();
            std::cout << "Quitting..." << std::endl;
            fs::remove( gx2osd::get_config().get_pid_file() );

        }
        if( gx2osd::get_config().show_conf() ) {
            gx2osd::conf_window conf;
            Gtk::Main::run( conf.get_window() );
        }

    }
    catch( const Gnome::Glade::XmlError& e ) {
        std::cout << "Error loading glade file: " << e.what() << std::endl;
    }
    catch( const std::exception& e ) {
        std::cout << e.what() << std::endl;
    }

}
