/* main_window.hh: OSD Window header.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#ifndef GX2OSD_MAIN_WINDOW_HH
#define GX2OSD_MAIN_WINDOW_HH

#include <gtkmm/window.h>
#include <gtkmm/label.h>
#include <gtkmm/fixed.h>
#include <gtkmm/image.h>

#include <glibmm/ustring.h>

#include <xmmsclient/xmmsclient++.h>

#include "skin.hh"
#include "scroller.hh"

#include "boost/noncopyable.hpp"
#include "boost/shared_ptr.hpp"

namespace gx2osd {

    class main_window : public Gtk::Window, private boost::noncopyable {

        public:

            main_window();
            virtual ~main_window();

            void set_text( const Glib::ustring& artist,
                           const Glib::ustring& title,
                           const Glib::ustring& album );
            void set_text_and_show( const Glib::ustring& artist,
                                    const Glib::ustring& title,
                                    const Glib::ustring& album );

            void set_cover( const Xmms::bin& cover,
                            const Glib::ustring& artist,
                            const Glib::ustring& title,
                            const Glib::ustring& album );

            bool set_status( const Xmms::Playback::Status& status );
            bool set_status_no_show( const Xmms::Playback::Status& status );

            void show_all();

        private:
            void load_skin();
            void set_label_stuff( eclidgets::scroller& gtklabel,
                                  const gx2osd::label& label,
                                  Glib::ustring& markup );
            bool hider();
            virtual bool on_button_press_event( GdkEventButton* event );

            Glib::RefPtr< Gdk::Pixbuf >
            scale_pixbuf( Glib::RefPtr< Gdk::Pixbuf > const& pixbuf ) const;

            eclidgets::scroller artist_name_;
            eclidgets::scroller song_name_;
            eclidgets::scroller album_name_;
            Gtk::Fixed fixed_pane_;

            Glib::ustring artist_markup_;
            Glib::ustring song_name_markup_;
            Glib::ustring album_name_markup_;

            const Glib::ustring span_end_;

            Gtk::Image background_;
            Gtk::Image cover_;
            Gtk::Image status_;

            const boost::shared_ptr< gx2osd::skin >& skin_;

    };

    main_window& get_window();

}

#endif // GX2OSD_MAIN_WINDOW_HH
