/* scroller.cc: Scrolling text widget header
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#ifndef ECLIPSERS_WIDGET_SCROLLER_HH
#define ECLIPSERS_WIDGET_SCROLLER_HH

#include <gtkmm/widget.h>
#include <pangomm/layout.h>
#include <glibmm/ustring.h>

namespace eclidgets
{

    class scroller : public Gtk::Widget
    {

        public:

            enum scrolling
            {
                SCROLL_SMART,
                SCROLL_ALWAYS,
                SCROLL_NEVER
            };

            enum scroll_direction
            {
                SCROLL_TO_LEFT = -1,
                SCROLL_TO_RIGHT = 1
            };

            scroller( Glib::ustring const& text = "",
                      scrolling scroll = SCROLL_SMART,
                      scroll_direction direction = SCROLL_TO_LEFT,
                      Gtk::Justification default_justify = Gtk::JUSTIFY_CENTER,
                      unsigned int interval = 50, unsigned int multiplier = 1,
                      unsigned int wait_before_scroll = 1000,
                      int gap = 50 );

            virtual ~scroller();

            Glib::ustring
            get_text() const;

            scrolling
            get_scroll() const;

            scroll_direction
            get_direction() const;

            Gtk::Justification
            get_justification() const;

            void
            get_speed( unsigned int& interval, unsigned int& multiplier ) const;

            int
            get_gap() const;

            unsigned int
            get_wait_before_scroll() const;

            Pango::FontDescription
            get_font_description() const;

            void
            set_text( Glib::ustring const& str );

            void
            set_markup( Glib::ustring const& markup );

            void
            set_scroll( scrolling scroll );

            void
            set_direction( scroll_direction direction );

            void
            set_justification( Gtk::Justification justify );

            void
            set_speed( unsigned int interval, unsigned int multiplier = 1 );

            void
            set_gap( int gap );

            void
            set_wait_before_scroll( unsigned int wait_before_scroll );

            void
            set_width_chars( unsigned int chars );

            void
            set_font_description( Pango::FontDescription const& desc );

        protected:

            virtual void
            on_realize();

            virtual bool
            on_expose_event( GdkEventExpose* event );

            virtual void
            on_size_allocate( Gtk::Allocation& allocation );

        private:

            Glib::RefPtr< Pango::Layout > layout_;
            Glib::RefPtr< Gdk::GC > gc_;

            int pos_;
            Gtk::Justification justify_;
            scrolling scroll_;
            scroll_direction direction_;

            bool scrolling_;
            int width_;

            unsigned int interval_;
            unsigned int multiplier_;
            unsigned int wait_before_scroll_;
            int gap_;

            sigc::connection conn_;
            sigc::connection wait_;

            bool
            timer();

            void
            determine_scroll();

            bool
            start_scroll();

            void
            text_changed();

    };

}

#endif // ECLIPSERS_WIDGET_SCROLLER_HH
